-- Module
local BaikItemLevel = LibStub("AceAddon-3.0"):GetAddon("BaikItemLevel")
local Inspect = BaikItemLevel:NewModule("Inspect", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")
local ItemLevelFrame = BaikItemLevel.ItemLevelFrame
local ItemHelper = BaikItemLevel.ItemHelper

-- Constants
local ROOT_FRAME = nil
local FRAMES = nil

-- Flags
local isLoaded = false

local function GetTargetItemLevel(slot)
    if InspectFrame.unit then
        local itemLink = GetInventoryItemLink(InspectFrame.unit, slot)

        return ItemHelper:GetItemLevel(itemLink)
    else
        return nil, true
    end
end

function Inspect:Setup()
    isLoaded = true
    ROOT_FRAME = _G["InspectPaperDollItemsFrame"]
    FRAMES = {
        [INVSLOT_HEAD] = _G["InspectHeadSlot"],
        [INVSLOT_NECK] = _G["InspectNeckSlot"],
        [INVSLOT_SHOULDER] = _G["InspectShoulderSlot"],
        [INVSLOT_BODY] = _G["InspectShirtSlot"],
        [INVSLOT_CHEST] = _G["InspectChestSlot"],
        [INVSLOT_WAIST] = _G["InspectWaistSlot"],
        [INVSLOT_LEGS] = _G["InspectLegsSlot"],
        [INVSLOT_FEET] = _G["InspectFeetSlot"],
        [INVSLOT_WRIST] = _G["InspectWristSlot"],
        [INVSLOT_HAND] = _G["InspectHandsSlot"],
        [INVSLOT_FINGER1] = _G["InspectFinger0Slot"],
        [INVSLOT_FINGER2] = _G["InspectFinger1Slot"],
        [INVSLOT_TRINKET1] = _G["InspectTrinket0Slot"],
        [INVSLOT_TRINKET2] = _G["InspectTrinket1Slot"],
        [INVSLOT_BACK] = _G["InspectBackSlot"],
        [INVSLOT_MAINHAND] = _G["InspectMainHandSlot"],
        [INVSLOT_OFFHAND] = _G["InspectSecondaryHandSlot"],
        [INVSLOT_TABARD] = _G["InspectTabardSlot"]
    }
    self.itemLevelFrame = ItemLevelFrame.new("Baik_InspectFrame", InspectFrame,
                                             FRAMES, GetTargetItemLevel)
    if ROOT_FRAME:IsShown() then
        self:Load()
    else
        self:HookScript(ROOT_FRAME, "OnShow", "Load")
    end
    self:RegisterEvent("UNIT_INVENTORY_CHANGED")
    self:RegisterEvent("PLAYER_AVG_ITEM_LEVEL_UPDATE")
    self:RegisterEvent("INSPECT_READY")
end

function Inspect:Load()
    if self:IsHooked(ROOT_FRAME, "OnShow") then
        self:Unhook(ROOT_FRAME, "OnShow")
    end
    self.itemLevelFrame:Update()
    if self.itemLevelFrame:HasMissing() and not self.callback then
        self.callback = self:ScheduleTimer("LoadMissing", 5)
    end
end

function Inspect:LoadMissing()
    self.itemLevelFrame:UpdateMissing()
    if self.itemLevelFrame:HasMissing() then
        self.callback = self:ScheduleTimer("LoadMissing", 5)
    else
        self.callback = nil
    end
end

function Inspect:OnUpdate()
    if self.itemLevelFrame then
        self:Load()
    end
end

-- Event Callbacks
function Inspect:UNIT_INVENTORY_CHANGED(opt, unitId)
    if InspectFrame.unit == unitId then
        self:Load()
    end
end

function Inspect:PLAYER_AVG_ITEM_LEVEL_UPDATE()
    self.itemLevelFrame:UpdateColor()
end

function Inspect:INSPECT_READY()
    self:Load()
end

function Inspect:ADDON_LOADED()
    if IsAddOnLoaded("Blizzard_InspectUI") then
        self:Setup()
        self:UnregisterEvent("ADDON_LOADED")
    end
end

-- Ace3 Callbacks
function Inspect:OnInitialize()
    self.itemLevelFrame = nil
    BaikItemLevel:DPrint("Inspect OnInitialize")
end

function Inspect:OnEnable()
    if IsAddOnLoaded("Blizzard_InspectUI") then
        self:Setup()
    else
        self:RegisterEvent("ADDON_LOADED")
    end

    BaikItemLevel:DPrint("Inspect OnEnable")
end

function Inspect:OnDisable()
    BaikItemLevel:DPrint("Inspect OnDisable")
end
