-- Module
local BaikItemLevel = LibStub("AceAddon-3.0"):GetAddon("BaikItemLevel")
local Inventory = BaikItemLevel:NewModule("Inventory", "AceEvent-3.0",
                                          "AceTimer-3.0", "AceHook-3.0")
local ItemHelper = BaikItemLevel.ItemHelper
local ItemLevelFrame = BaikItemLevel.ItemLevelFrame

-- Constants
local ROOT_FRAME = PaperDollFrame
local FRAMES = {
    [INVSLOT_HEAD] = CharacterHeadSlot,
    [INVSLOT_NECK] = CharacterNeckSlot,
    [INVSLOT_SHOULDER] = CharacterShoulderSlot,
    [INVSLOT_BODY] = CharacterShirtSlot,
    [INVSLOT_CHEST] = CharacterChestSlot,
    [INVSLOT_WAIST] = CharacterWaistSlot,
    [INVSLOT_LEGS] = CharacterLegsSlot,
    [INVSLOT_FEET] = CharacterFeetSlot,
    [INVSLOT_WRIST] = CharacterWristSlot,
    [INVSLOT_HAND] = CharacterHandsSlot,
    [INVSLOT_FINGER1] = CharacterFinger0Slot,
    [INVSLOT_FINGER2] = CharacterFinger1Slot,
    [INVSLOT_TRINKET1] = CharacterTrinket0Slot,
    [INVSLOT_TRINKET2] = CharacterTrinket1Slot,
    [INVSLOT_BACK] = CharacterBackSlot,
    [INVSLOT_MAINHAND] = CharacterMainHandSlot,
    [INVSLOT_OFFHAND] = CharacterSecondaryHandSlot,
    [INVSLOT_TABARD] = CharacterTabardSlot
}

local function GetPlayerInventorySlotLevel(slot)
    local itemLink = GetInventoryItemLink(PLAYER, slot)

    return ItemHelper:GetItemLevel(itemLink)
end

function Inventory:Load()
    if self:IsHooked(ROOT_FRAME, "OnShow") then
        self:Unhook(ROOT_FRAME, "OnShow")
    end
    self.itemLevelFrame:Update()
    if self.itemLevelFrame:HasMissing() and not self.callback then
        self.callback = self:ScheduleTimer("LoadMissing", 5)
    end
end

function Inventory:LoadMissing()
    self.itemLevelFrame:UpdateMissing()
    if self.itemLevelFrame:HasMissing() then
        self.callback = self:ScheduleTimer("LoadMissing", 5)
    else
        self.callback = nil
    end
end

function Inventory:OnUpdate()
    self.itemLevelFrame:Update()
end

-- Event Callbacks
function Inventory:PLAYER_EQUIPMENT_CHANGED(opt, slot, hasItem)
    self.itemLevelFrame:SetSlot(slot)
end

function Inventory:PLAYER_AVG_ITEM_LEVEL_UPDATE()
    self.itemLevelFrame:UpdateColor()
end

-- Ace3 Callbacks
function Inventory:OnInitialize()
    self.itemLevelFrame = ItemLevelFrame.new("Baik_InventoryFrame", ROOT_FRAME,
                                             FRAMES, GetPlayerInventorySlotLevel)
    BaikItemLevel:DPrint("Inventory OnInitialize")
end

function Inventory:OnEnable()
    self:HookScript(ROOT_FRAME, "OnShow", "Load")
    self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
    self:RegisterEvent("PLAYER_AVG_ITEM_LEVEL_UPDATE")
    BaikItemLevel:DPrint("Inventory OnEnable")
end

function Inventory:OnDisable()
    BaikItemLevel:DPrint("Inventory OnDisable")
end
