local BaikItemLevel = LibStub("AceAddon-3.0"):GetAddon("BaikItemLevel")
local ItemHelper = BaikItemLevel.ItemHelper
local ItemLevelBadge = {}

-- Add Module to Addon
ItemLevelBadge.__index = ItemLevelBadge
BaikItemLevel.ItemLevelBadge = ItemLevelBadge

-- Constants
ItemLevelBadge.INVALID = -1

-- Creates new object
function ItemLevelBadge.new(name, parentFrame)
    local self = setmetatable({}, ItemLevelBadge)

    local fontString = parentFrame:CreateFontString(name, "OVERLAY", "NumberFontNormal")
    fontString:SetPoint("BOTTOM", 0, 3)
    self.fontString = fontString
    self.itemLevel = nil
    self.itemQuality = nil

    return self
end

-- ItemLevelBadge API
function ItemLevelBadge:UpdateColor()
    if BaikItemLevel.db.profile.colorLevelSimple and self.itemQuality then
        local red, blue, green = GetItemQualityColor(self.itemQuality)
        self.fontString:SetTextColor(red, blue, green)
    else
        local itemLevel = self.itemLevel

        if itemLevel and itemLevel ~= self.INVALID then
            local quality = ItemHelper:GetQualityColor(itemLevel)
            local red, blue, green = GetItemQualityColor(quality)

            self.fontString:SetTextColor(red, blue, green)
        end
    end
end

function ItemLevelBadge:SetLevel(itemLevel, quality)
    if itemLevel ~= self.INVALID then
        self.itemQuality = quality
        self.itemLevel = itemLevel
        self.fontString:SetText(itemLevel)
        self:UpdateColor()
    else
        self.itemQuality = nil
        self.fontString:SetTextColor(0.85, 0.209, 0.106, 1)
        self.fontString:SetText("load")
    end

    if itemLevel <= BaikItemLevel.db.profile.filterLevel and itemLevel ~= self.INVALID then
        self.fontString:Hide()
    else
        self.fontString:Show()
    end
end

function ItemLevelBadge:Hide()
    self.fontString:Hide()
end
